@@ -1,4 +1,4 @@
-/* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 8-*- */
+/* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
 /*
  *This file is part of MlView.
  *
@@ -32,14 +32,15 @@
 #include <string.h>
 #include <libxml/parser.h>
 #include <libxml/catalog.h>
+#include <libxml/valid.h>
 #include <libxml/parserInternals.h>
 #include <libxml/xmlmemory.h>
 #include <libxml/SAX.h>
-#include <libxml/xmlerror.h>
 #include <libxml/uri.h>
-#include <libxml/hash.h>
+/*#include <libxml/hash.h>*/
 #include <glade/glade.h>
 #include <libgnome/libgnome.h>
+#include <libgnomevfs/gnome-vfs.h>
 #include <zlib.h>
 
 #include "mlview-utils.h"
@@ -57,22 +58,6 @@ static const gint MAX_NUMBER_OF_ELEMENT_NAMES_IN_CHOICE_LIST = 256;
 
 static GtkWidget *gv_dtd_choice_dialog = NULL;
 
-/*
-#ifdef MLVIEW_DATA_DIR
-static const gchar 
-* gv_mlview_data_dir = MLVIEW_DATA_DIR ;
-#else
-static const gchar * gv_mlview_data_dir = NULL ;
-#endif
-
-#ifdef MLVIEW_XML_CATALOG_DIR_NAME
-static const gchar * gv_mlview_xml_catalog_dir_name = 
-MLVIEW_XML_CATALOG_DIR_NAME ;
-#else
-static const gchar * gv_mlview_xml_catalog_dir_name = NULL ;
-#endif
-*/
-
 typedef struct _MlViewParserCtxt MlViewParserCtxt;
 
 /**
@@ -115,7 +100,9 @@ static xmlDtdPtr custom_xmlSAXParseDTD (MlViewAppContext * a_app_context,
 
 
 static gboolean mlview_parsing_utils_ask_for_DTD_change_and_validation (MlViewAppContext * a_app_context,
-                                                                        const gchar * a_external_id, const gchar * a_system_id,
+                                                                        const gchar * a_external_id, 
+                                                                        const gchar * a_system_id,
+                                                                        const xmlDoc *a_doc,
                                                                         MlViewExtSubsDef ** a_ext_subs_def);
 
 static xmlChar *mlview_resolve_external_entity (MlViewAppContext * a_app_ctxt,
@@ -182,7 +169,6 @@ mlview_ext_subs_def_clone (MlViewExtSubsDef * a_ext_subs_def)
 
 /**
  *destroys the MlViewExtSubsDef given in argument. 
- *
  */
 void
 mlview_ext_subs_def_destroy (MlViewExtSubsDef * a_def) 
@@ -225,18 +211,16 @@ mlview_ext_subs_def_destroy (MlViewExtSubsDef * a_def)
  *to report  MLVIEW_ERRORS 
  */
 static enum MlViewStatus
-load_xml_document_from_local_file (gchar * a_xml_file_name,
+load_xml_document_from_local_file (gchar * a_xml_file_uri,
                                    xmlParserCtxt ** a_parser_context,
                                    gboolean a_store_external_subset_info,
                                    MlViewAppContext * a_app_context) 
 {
-        FILE *file_ptr = NULL;
-        int compressed = 0, c1 = 0, c2 = 0;
-        gzFile zfile_ptr = NULL;
-        int size = 1024,
-                num_of_chars_read = 0,
-                parse_status = MLVIEW_OK;
-        gchar file_buffer[size] ;
+        GnomeVFSResult vfs_result = GNOME_VFS_OK;
+        GnomeVFSHandle *vfs_handle;
+        GnomeVFSFileSize  bytes = 1024, read_bytes;
+        gchar buffer[bytes];
+        int parse_status = MLVIEW_OK;
         xmlParserCtxt *xml_ctxt = NULL;
         struct MlViewAppSettings *settings = NULL ;
 
@@ -249,47 +233,30 @@ load_xml_document_from_local_file (gchar * a_xml_file_name,
         xmlDoValidityCheckingDefaultValue = 0;
 
         if (a_app_context) {
-                settings = mlview_app_context_get_settings
-                        (a_app_context) ;
+                settings = mlview_app_context_get_settings (a_app_context) ;
                 g_return_val_if_fail (settings, MLVIEW_ERROR) ;
         }
 
         /*now, open the xml file... */
-        g_return_val_if_fail (a_xml_file_name != NULL,
+        g_return_val_if_fail (a_xml_file_uri != NULL,
                               MLVIEW_NULL_FILE_NAME_ERROR);
 
-        file_ptr = fopen (a_xml_file_name, "rb");
-        if (!file_ptr)
-                return MLVIEW_BAD_FILE_ERROR ;
 
-        c1 = fgetc (file_ptr);
-        c2 = fgetc (file_ptr);
-        fclose (file_ptr);
-        if ((c1 == 0x1F) && (c2 == 0x8B)) {
-                compressed = 1;
-        } else {
-                compressed = 0;
-        }
+        vfs_result = gnome_vfs_open (&vfs_handle, a_xml_file_uri, GNOME_VFS_OPEN_READ);
+        g_return_val_if_fail (vfs_result == GNOME_VFS_OK, MLVIEW_ERROR);
 
-        zfile_ptr = gzopen (a_xml_file_name, "rb");
-
-        g_return_val_if_fail (zfile_ptr != NULL,
-                              MLVIEW_BAD_FILE_ERROR);
-
-        num_of_chars_read = gzread (zfile_ptr, file_buffer, 4);
-        g_return_val_if_fail (num_of_chars_read > 0,
-                              MLVIEW_EMPTY_FILE_ERROR);
+        /* Read first four bytes to create the parser context */
+        vfs_result = gnome_vfs_read (vfs_handle, (gpointer)buffer, (GnomeVFSFileSize)4, &read_bytes);
+        g_return_val_if_fail (vfs_result == GNOME_VFS_OK, MLVIEW_ERROR);
+        
         /*
          *Force the parser to ignore ignorable white spaces so that
          *indentation can be made properly at save time.
          */
         xmlKeepBlanksDefault (0);
         gv_app_ctxt = a_app_context;
-        xml_ctxt =
-                xmlCreatePushParserCtxt (NULL, NULL,
-                                         file_buffer,
-                                         num_of_chars_read,
-                                         a_xml_file_name);
+        xml_ctxt = xmlCreatePushParserCtxt (NULL, NULL, buffer, (int)read_bytes, a_xml_file_uri);
+
         if (a_store_external_subset_info == TRUE) {
                 /*
                  *overload the externalSubset sax handler with our custom one
@@ -301,6 +268,9 @@ load_xml_document_from_local_file (gchar * a_xml_file_name,
                 /*overload the externalSubset sax handler */
                 xml_ctxt->sax->externalSubset =
                         mlview_external_subset_sax_handler;
+
+                xml_ctxt->sax->resolveEntity =
+                        mlview_sax_resolve_entity;
                 /*
                  *tell the custom sax handler to save the 
                  *external subset definition ... or not
@@ -312,16 +282,20 @@ load_xml_document_from_local_file (gchar * a_xml_file_name,
         xmlSetGenericErrorFunc (a_app_context,
                                 (xmlGenericErrorFunc)
                                 mlview_app_context_bufferize_error);
-        while ((num_of_chars_read =
-                gzread (zfile_ptr, file_buffer, size)) > 0) {
-                parse_status =
-                        xmlParseChunk
-                        (xml_ctxt,
-                         file_buffer, num_of_chars_read, 0);
 
-                if (parse_status != XML_ERR_OK)
+        while (vfs_result == GNOME_VFS_OK) {
+                vfs_result = gnome_vfs_read (vfs_handle, buffer, bytes, &read_bytes);
+                if (vfs_result != GNOME_VFS_OK)
                         break;
+
+                if (read_bytes > 0) {
+                        parse_status = xmlParseChunk (xml_ctxt, (char*)buffer, (int) read_bytes, 0);
+
+                        if (parse_status != XML_ERR_OK)
+                                break;
+                }
         }
+        
         /*
          *call the xmlParseChunk once more 2 
          *consume all the data stored in 
@@ -329,15 +303,13 @@ load_xml_document_from_local_file (gchar * a_xml_file_name,
          */
         if (parse_status == XML_ERR_OK) {
                 parse_status =
-                        xmlParseChunk (xml_ctxt, file_buffer,
-                                       num_of_chars_read, 1);
+                        xmlParseChunk (xml_ctxt, buffer,(int)read_bytes, 1);
         }
         *a_parser_context = xml_ctxt;
         /*restore the error display stream of libxml */
         xmlSetGenericErrorFunc (NULL, NULL);
         if (a_app_context
-            &&
-            !mlview_app_context_error_buffer_is_empty
+            && !mlview_app_context_error_buffer_is_empty
             (a_app_context)) {
                 mlview_app_context_display_buffered_error
                         (a_app_context);
@@ -345,19 +317,23 @@ load_xml_document_from_local_file (gchar * a_xml_file_name,
                 mlview_app_context_set_error_dialog_title
                         (a_app_context, NULL);
         }
-        gzclose (zfile_ptr);
+
+        /* close file */
+        vfs_result = gnome_vfs_close (vfs_handle);
+
         /*
          * If the file is compressed ... we set it inside the doc
-         */
+        
         if (compressed) {
                 printf ("Compression detected\n");
                 xmlSetDocCompressMode ((**a_parser_context).
                                        myDoc, 9);
         }
+        */
+        
         return (parse_status == XML_ERR_OK) ? 0 : parse_status;
 }
 
-
 /**
  *A custom entity loader that first fetches 
  *the entities by resolving their url reference
@@ -470,6 +446,12 @@ mlview_resolve_external_entity (MlViewAppContext * a_app_context,
                               (a_app_context), FALSE);
 
         result = xmlCatalogResolve (a_pub_id, a_sys_id);
+        if (!result && a_sys_id) {
+                if (g_file_test (a_sys_id, G_FILE_TEST_EXISTS) == TRUE
+                    && g_file_test (a_sys_id, G_FILE_TEST_IS_DIR) == FALSE) {
+                        result = g_strdup (a_sys_id) ;
+                }
+        }
         return result;
 }
 
@@ -494,7 +476,7 @@ mlview_external_subset_sax_handler (void *a_ctxt,
 {
         xmlParserCtxt *ctxt = NULL;
         struct MlViewAppSettings *settings = NULL ;
-
+        gchar *system_id = NULL ;
         g_return_if_fail (a_ctxt);
 
         ctxt = a_ctxt;
@@ -514,6 +496,7 @@ mlview_external_subset_sax_handler (void *a_ctxt,
         } else {
                 ctxt->validate = 0;
         }
+        system_id = (gchar*)a_system_id ;
 
         /*
          *If validation is on, ask the user if she wants 
@@ -527,7 +510,7 @@ mlview_external_subset_sax_handler (void *a_ctxt,
                 gboolean validate =
                         mlview_parsing_utils_ask_for_DTD_change_and_validation
                         (gv_app_ctxt, a_external_id,
-                         a_system_id, &ext_subs_def);
+                         system_id, ctxt->myDoc, &ext_subs_def);
 
                 if (validate == FALSE)
                         ctxt->validate = 0;
@@ -535,35 +518,37 @@ mlview_external_subset_sax_handler (void *a_ctxt,
                 if (validate == TRUE && ext_subs_def) {
                         a_external_id =
                                 ext_subs_def->external_id;
-
-                        a_system_id =
+                        system_id =
                                 ext_subs_def->system_id;
                 }
         }
 
         if (ctxt->validate == 1) {
+#ifdef LIBXML_2_6_SERIES_OR_ABOVE
+                xmlSAX2ExternalSubset (ctxt,
+                                       a_name, a_external_id,
+                                       system_id) ;
+#else
                 externalSubset (ctxt,
                                 a_name, a_external_id,
-                                a_system_id);
-
+                                system_id);
+#endif /*LIBXML_2_6_SERIES_OR_ABOVE*/
                 if (ctxt->myDoc
                     && (ctxt->myDoc->extSubset == NULL)) {
                         mlview_app_context_warning
                                 (gv_app_ctxt,
-                                 _
-                                 ("The external DTD subset was not found. I couldn't validate the document."));
+                                 _("The external DTD subset was not found. I couldn't validate the document."));
                 }
+
+                /*
+                 *We will not do validation during the document parsing.
+                 *We will only do post parsing validation 
+                 *(if validation is required).
+                 */
+                ctxt->validate = 0;
         }
-
-        /*
-         *We will not do validation during the document parsing.
-         *We will only do post parsing validation 
-         *(if validation is required).
-         */
-        ctxt->validate = 0;
 }
 
-
 /**
  *This code has been copied and modified from libxml2.
  *GPL code is good for you.
@@ -670,7 +655,7 @@ custom_xmlSAXParseDTD (MlViewAppContext * a_app_context,
 
         xmlFreeParserCtxt (ctxt);
 
-        return (ret);
+                return (ret);
 }
 
 /**
@@ -693,32 +678,80 @@ static gboolean
 mlview_parsing_utils_ask_for_DTD_change_and_validation (MlViewAppContext * a_app_context,
                                                         const gchar * a_external_id,
                                                         const gchar * a_system_id,
+                                                        const xmlDoc *a_doc,
                                                         MlViewExtSubsDef ** a_ext_subs_def)
 {
         GladeXML *gxml = NULL;
         gchar *gfile = NULL ;
         gint button = 0;
-        gboolean result = TRUE;
+        gboolean result = TRUE, is_relative = FALSE;
         GtkWidget *label_public_id = NULL;
         GtkWidget *label_system_id = NULL;
         GtkWidget *label_resource = NULL;
-        const gchar *public_id = NULL;
-        const gchar *system_id = NULL;
+        gchar *public_id = NULL;
+        gchar *system_id = NULL;
         xmlChar *resource = NULL;
+        gchar *base_uri = NULL;
+        gchar *tmp_str = NULL;
+        enum MlViewStatus status = MLVIEW_BAD_PARAM_ERROR;
 
         g_return_val_if_fail (a_app_context != NULL, TRUE);
         g_return_val_if_fail (a_ext_subs_def != NULL,
                               TRUE);
 
-        public_id = ((const gchar *) (a_external_id != NULL)) ?
-                a_external_id : "\"\"";
-        system_id = ((const gchar *) (a_system_id != NULL)) ?
-                a_system_id : "\"\"";
+        if (a_external_id != NULL)
+                public_id = (gchar*)a_external_id ;
+        else 
+                public_id = (gchar*)"\"\"";
+
+        if (a_system_id != NULL) 
+                system_id = (gchar*)a_system_id ;
+        else 
+                system_id = (gchar*)"\"\"";
+
         *a_ext_subs_def = NULL; /*sanity initialization */
+        system_id = g_strdup (system_id) ;
+
+        mlview_utils_uri_is_relative (system_id, &is_relative) ;
+
+        if (is_relative == TRUE) { 
+                base_uri = mlview_utils_get_dir_name_from_uri (a_doc->URL);
+
+                if (!base_uri) {
+                        mlview_utils_trace_info ("Can't get base URI.");
+
+                        g_free (system_id);
+
+                        system_id = (gchar*)"\"\"";
+                } else {
+                        status = mlview_utils_relative_uri_to_absolute_uri (system_id,
+                                                                            base_uri,
+                                                                            &tmp_str);
+                        
+                        g_free (base_uri);
+                        base_uri = NULL;
+
+                        if (!tmp_str || status != MLVIEW_OK) {
+                                mlview_utils_trace_info ("Can't get absolute URI.");
+
+                                g_free (system_id);
+                                
+                                system_id = (gchar*)"\"\"";
+                        }
+                        else {
+                                g_free (system_id);
+
+                                system_id = tmp_str;
+                                
+                                tmp_str = NULL;
+                        }
+                }
+        }
+
         resource =
                 mlview_resolve_external_entity (a_app_context,
                                                 a_external_id,
-                                                a_system_id);
+                                                system_id);
         if (resource) {
                 gfile = gnome_program_locate_file
                         (NULL, GNOME_FILE_DOMAIN_APP_DATADIR,
@@ -853,11 +886,12 @@ mlview_parsing_utils_ask_for_DTD_change_and_validation (MlViewAppContext * a_app
         gtk_widget_hide (GTK_WIDGET (gv_dtd_choice_dialog));
         g_object_unref (gxml);
         g_free (gfile);
-
+        if (system_id) {
+                g_free (system_id) ;
+        }
         return result;
 }
 
-
 /**
  *The callback of the libxml2 hash table scan method xmlHashScan () ;
  *Takes a_hash_data 
@@ -1107,7 +1141,11 @@ g_list_compare_string_elems (gchar * a_str1, gchar * a_str2)
  ******************************************/
 
 /**
- *Loads an xml file located on the disk.
+ *Loads an xml file located on the disk. The user not prompted
+ *at all, all informations about any DTD is forgotten.
+ *If no DTD name is specified or if there was an error while 
+ *loading the specified DTD, the returned document's extSubset
+ *will be NULL.
  *@param a_file_name the name of the file to open and parse.
  *@param a_app_context the application context to use. 
  *If set to NULL, 
@@ -1115,17 +1153,86 @@ g_list_compare_string_elems (gchar * a_str1, gchar * a_str2)
  *@return the xmlDoc resulting from 
  *the parsing or NULL if the parsing failed. 
  */
-xmlDoc *
-mlview_parsing_utils_load_xml_file (gchar * a_file_name,
-                                    MlViewAppContext *
-                                    a_app_context)
+xmlDocPtr
+mlview_parsing_utils_load_xml_file_with_dtd (gchar *a_file_name,
+                                             gchar *a_dtd_name,
+                                             MlViewAppContext *a_app_context)
+{
+        xmlDoc *result = NULL;
+        xmlParserCtxtPtr parser_context = NULL;
+        xmlDtdPtr dtd = NULL;
+        
+        g_return_val_if_fail (a_file_name != NULL, NULL);
+        
+        if (strcmp (a_file_name, "")) {
+                gint load_res;
+                
+                /*
+                 *fixme: remember to add support of 
+                 *error handling while loading the document.
+                 */
+                load_res =
+                        load_xml_document_from_local_file
+                        (a_file_name, &parser_context,
+                         FALSE, a_app_context);
+                
+                if (parser_context == NULL) {
+                        mlview_app_context_error
+                                (a_app_context,
+                                  _
+                                 ("could not load xml document %s"),
+                                 a_file_name);
+                        
+                        return NULL;
+                }
+                
+                if (!load_res) {
+                        parser_context->myDoc->name = (xmlChar *)
+                                g_strdup (a_file_name);
+                        result = parser_context->myDoc;
+                }
+        }
+        
+        if (result && a_dtd_name) {
+                dtd = mlview_parsing_utils_load_dtd (a_dtd_name,
+                                                     a_app_context);
+                
+                if (dtd) {
+                        result->standalone = TRUE;
+                        result->extSubset = dtd;
+                        
+                        dtd->doc = result;
+                }
+        }
+        
+        if (parser_context != NULL) {
+                xmlFreeParserCtxt (parser_context);
+        }
+        
+        return result;
+}
+ 
+/**
+ *Loads an xml file located on the disk. The user is prompted
+ *for informations if a DTD is specified in the document.
+ *Loads an xml file from an URI.
+ *@param a_file_uri the URI of the file to open and parse.
+ *@param a_app_context the application context to use. 
+ *If set to NULL, 
+ *it is not taken in account.
+ *@return the xmlDoc resulting from 
+ *the parsing or NULL if the parsing failed. 
+ */
+xmlDocPtr 
+mlview_parsing_utils_load_xml_file_with_dtd_interactive (gchar *a_file_uri,
+                                                         MlViewAppContext *a_app_context)
 {
         xmlDoc *result = NULL;
         xmlParserCtxtPtr parser_context = NULL;
 
-        g_return_val_if_fail (a_file_name != NULL, NULL);
+        g_return_val_if_fail (a_file_uri != NULL, NULL);
 
-        if (strcmp (a_file_name, "")) {
+        if (strcmp (a_file_uri, "")) {
                 gint load_res;
 
                 /*
@@ -1134,7 +1241,7 @@ mlview_parsing_utils_load_xml_file (gchar * a_file_name,
                  */
                 load_res =
                         load_xml_document_from_local_file
-                        (a_file_name, &parser_context,
+                        (a_file_uri, &parser_context,
                          TRUE, a_app_context);
 
                 if (parser_context == NULL) {
@@ -1142,14 +1249,14 @@ mlview_parsing_utils_load_xml_file (gchar * a_file_name,
                                 (a_app_context,
                                  _
                                  ("could not load xml document %s"),
-                                 a_file_name);
+                                 a_file_uri);
 
                         return NULL;
                 }
 
                 if (!load_res) {
                         parser_context->myDoc->name = (xmlChar *)
-                                g_strdup (a_file_name);
+                                g_strdup (a_file_uri);
                         result = parser_context->myDoc;
                 }
         }
@@ -1160,7 +1267,6 @@ mlview_parsing_utils_load_xml_file (gchar * a_file_name,
         return result;
 }
 
-
 /**
  *Saves an xml file using the application wide settings. 
  *@param a_xml_doc the xml document to save.
@@ -1178,7 +1284,6 @@ mlview_parsing_utils_save_xml_doc (xmlDoc * a_xml_doc,
         return xmlSaveFormatFile (a_file_path, a_xml_doc, 1);
 }
 
-
 /**
  *Loads an external DTD subset entity.
  *@param a_subset_def the definition of the external DTD subset to load.
@@ -1207,11 +1312,12 @@ mlview_parsing_utils_load_a_dtd (MlViewExtSubsDef * a_subset_def,
                 g_assert (sax_handler != NULL);
 
                 memset (sax_handler, 0, sizeof (xmlSAXHandler));
-
+#ifdef LIBXML_2_6_SERIES_OR_ABOVE
+                xmlSAX2InitDefaultSAXHandler(sax_handler, 0) ;
+#else
                 initxmlDefaultSAXHandler (sax_handler, 0);
-
+#endif
                 g_assert (sax_handler != NULL);
-
                 /*override the libxml error printing routine */
                 xmlSetGenericErrorFunc
                         (a_app_context, (xmlGenericErrorFunc)
@@ -1271,7 +1377,7 @@ mlview_parsing_utils_validate_dtd (xmlDoc * a_doc,
 {
         gint result = 0;
         gint validity_status;
-        xmlValidCtxt validation_context;
+        xmlValidCtxt validation_context = {0};
 
         g_return_val_if_fail (a_doc != NULL, -1);
 
@@ -1281,9 +1387,11 @@ mlview_parsing_utils_validate_dtd (xmlDoc * a_doc,
         validation_context.warning = (xmlValidityWarningFunc)
                 mlview_app_context_bufferize_error;
 
+
         xmlSetGenericErrorFunc (a_app_context,
                                 (xmlGenericErrorFunc)
                                 mlview_app_context_bufferize_error);
+        
         mlview_app_context_set_error_dialog_title
                 (a_app_context,
                  _
@@ -1293,10 +1401,6 @@ mlview_parsing_utils_validate_dtd (xmlDoc * a_doc,
                 xmlValidateDtd (&validation_context, a_doc,
                                 a_dtd);
 
-        xmlSetGenericErrorFunc
-                (a_app_context, (xmlGenericErrorFunc)
-                 mlview_app_context_bufferize_error);
-
         if (validity_status == 1) {
                 /*xmlValidate() returns 1 if doc is valid 
                  *whereas we want to return 0 in that case ...*/
@@ -1313,6 +1417,9 @@ mlview_parsing_utils_validate_dtd (xmlDoc * a_doc,
                 mlview_app_context_set_error_dialog_title
                         (a_app_context, NULL);
 
+        /*restore the generic error handling routine*/
+        xmlSetGenericErrorFunc (NULL, NULL) ;
+
         return result;
 }
 
@@ -1320,9 +1427,7 @@ mlview_parsing_utils_validate_dtd (xmlDoc * a_doc,
 /**
  *found during the parsing of the last xml document if and only if 
  *the parsing has been triggered by calling the
- *function mlview_parsing_utils_load_xml_file(gchar * a_file_name).
- *Note that after a given xml document parsing, 
- *this function can only be 
+ *function mlview_parsing_utils_load_xml_file_interactive_doctype.
  *
  *@return NULL. This function can also return NULL if no external 
  *subset has been declared in the 
@@ -1356,11 +1461,12 @@ MlViewExtSubsDef *
 mlview_parsing_utils_let_user_choose_a_dtd (MlViewAppContext * a_app_context,
                                             gchar * a_title) 
 {
-        MlViewFileSelection *filesel = NULL;
+        gchar *dtd = NULL;
         MlViewExtSubsDef *subset_def = NULL;
+#ifndef MLVIEW_WITH_GTK_FILE_CHOOSER
+        MlViewFileSelection *filesel = NULL;
 
         enum MLVIEW_SELECTED_BUTTON button_clicked;
-        gchar *dtd = NULL;
 
         g_return_val_if_fail (a_app_context != NULL, NULL);
 
@@ -1374,7 +1480,10 @@ mlview_parsing_utils_let_user_choose_a_dtd (MlViewAppContext * a_app_context,
         case OK_BUTTON:
                 dtd = g_strdup (gtk_file_selection_get_filename
                                 (GTK_FILE_SELECTION (filesel)));
-                if (dtd) {
+                if (dtd && strcmp (dtd, "") 
+                    && g_file_test (dtd, G_FILE_TEST_IS_DIR))
+                        break ;
+                if (dtd && strcmp (dtd, "")) {
                         subset_def =
                                 mlview_ext_subs_def_new (NULL, NULL, 
                                                          dtd);
@@ -1390,6 +1499,37 @@ mlview_parsing_utils_let_user_choose_a_dtd (MlViewAppContext * a_app_context,
                 break;
         }
 
+#else
+	gint response = 0;
+	GtkWidget *file_dialog = GTK_WIDGET (mlview_app_context_get_file_chooser 
+                                             (a_app_context, a_title)) ;
+
+        g_return_val_if_fail (file_dialog != NULL, NULL);
+
+        mlview_app_context_sbar_push_message
+                (a_app_context,
+		 _("Choose a dtd file"));
+
+	response = gtk_dialog_run (GTK_DIALOG(file_dialog));
+	gtk_window_set_modal (GTK_WINDOW (file_dialog), FALSE);
+	gtk_widget_hide (GTK_WIDGET (file_dialog));
+
+	if (response == GTK_RESPONSE_OK) {
+                dtd = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (file_dialog));
+		
+                if (dtd) {
+                        subset_def =
+                                mlview_ext_subs_def_new (NULL, NULL, 
+                                                         dtd);
+                }
+                if (dtd) {
+                        g_free (dtd);
+                        dtd = NULL;
+                }
+	}
+
+        mlview_app_context_sbar_pop_message (a_app_context);
+#endif
         return subset_def;
 }
 
@@ -1406,6 +1546,7 @@ mlview_parsing_utils_let_user_choose_a_dtd (MlViewAppContext * a_app_context,
  *relatively to the @a_current_xml_node.
  *@param a_app_context the current mlview application context.
  */
+#ifndef MLVIEW_WITH_LIBXML_EXPERIMENTAL_COMPLETION
 gint 
 mlview_parsing_utils_build_element_name_completion_list (MlViewAppContext * a_app_context,
                                                          enum NODE_INSERTION_SCHEME a_insertion_scheme,
@@ -1417,6 +1558,7 @@ mlview_parsing_utils_build_element_name_completion_list (MlViewAppContext * a_ap
         gint nb_of_names = 0;
         struct MlViewAppSettings *settings = NULL ;
 
+        xmlGenericError = NULL ;
         g_return_val_if_fail (a_current_xml_node != NULL, -2);
         g_return_val_if_fail (a_current_xml_node->type
                               == XML_ELEMENT_NODE, -2);
@@ -1503,6 +1645,14 @@ mlview_parsing_utils_build_element_name_completion_list (MlViewAppContext * a_ap
                 }
 
                 break;
+        case CHANGE_CUR_ELEMENT_NAME:
+                if (a_current_xml_node->prev && a_current_xml_node->next) {
+                        nb_of_names = xmlValidGetValidElements (a_current_xml_node->prev,
+                                                                a_current_xml_node->next,
+                                                                feasible_names,
+                                                                MAX_NUMBER_OF_ELEMENT_NAMES_IN_CHOICE_LIST) ;
+                }
+                break ;
         default:
                 break;
         }                       /*end switch */
@@ -1547,9 +1697,79 @@ mlview_parsing_utils_build_element_name_completion_list (MlViewAppContext * a_ap
                                      g_list_compare_string_elems);
         }
 
+        xmlSetGenericErrorFunc (NULL, NULL) ;
         return nb_of_names;
 }
+#else /*MLVIEW_WITH_LIBXML_EXPERIMENTAL_COMPLETION*/
+gint
+mlview_parsing_utils_build_element_name_completion_list (MlViewAppContext * a_app_context,
+                                                         enum NODE_INSERTION_SCHEME a_insertion_scheme,
+                                                         xmlNode * a_current_xml_node,
+                                                         GList ** a_feasible_names_ptr)
+{
+        
+        xmlCompletionListType  type  ;
+        xmlChar *element_names[50] = {0} ;
+        int ret = 0, len = 0 ;
+        GList *feasible_element_names = NULL ;
 
+        g_return_val_if_fail (a_app_context, -2) ;
+                
+        switch (a_insertion_scheme) {
+        case INSERT_BEFORE:
+                type = XML_PREV_SIBLING_LIST_COMPLETION_TYPE ;
+                break ;
+        case INSERT_AFTER:
+                type = XML_NEXT_SIBLING_LIST_COMPLETION_TYPE ;
+                break ;
+        case ADD_CHILD:
+                type =  XML_APPEND_CHILD_LIST_COMPLETION_TYPE ;
+                break ;
+        case CHANGE_CUR_ELEMENT_NAME:
+                type = XML_CURRENT_ELEMENT_LIST_COMPLETION_TYPE
+                break ;
+        case CHANGE_CUR_ELEMENT_NAME:
+                type = XML_CURRENT_ELEMENT_LIST_COMPLETION_TYPE ;
+                break ;
+        default :
+                ret = -1 ;
+                goto cleanup ;
+                break ;
+        }
+        if (!ret)
+                len = xmlValidGetElementNameCompletionList
+			(a_current_xml_node, NULL, type,
+			 (const xmlChar**)element_names,
+			 50) ;
+        else
+                goto cleanup ;
+
+        if (len) {
+                int i = 0 ;
+                for (i = 0 ; i < len ; i++) {
+                        feasible_element_names = g_list_prepend (feasible_element_names,
+                                                                 element_names[i]) ;
+                }
+                if (feasible_element_names) {
+                        g_list_reverse (feasible_element_names) ;
+                }
+        }
+        if (!ret) {                
+                ret = len ;
+                *a_feasible_names_ptr = g_list_concat (*a_feasible_names_ptr,
+                                                       feasible_element_names) ;
+                feasible_element_names = NULL ;
+        }
+
+ cleanup:
+        if (feasible_element_names) {
+                g_list_free (feasible_element_names) ;
+                feasible_element_names = NULL ;
+        }
+        return ret ;
+}
+
+#endif
 /**
  *Builds the atttribute completion list of a given xml document.
  *@param a_app_context the mlview application context.
@@ -2128,3 +2348,291 @@ mlview_parsing_utils_build_graphical_attr_values (MlViewAppContext * a_app_conte
 
         return result;
 }
+
+xmlDtdPtr
+mlview_parsing_utils_load_dtd (const gchar *a_url,
+                               MlViewAppContext *a_ctxt)
+{
+        MlViewExtSubsDef *sd = NULL;
+        xmlDtdPtr dtd = NULL;
+
+        g_return_val_if_fail (a_url, NULL);
+        g_return_val_if_fail (a_ctxt && 
+                              MLVIEW_IS_APP_CONTEXT (a_ctxt), 
+                              NULL);
+
+        sd = mlview_ext_subs_def_new (NULL, NULL, a_url);
+
+        if (!sd)
+                goto cleanup;
+
+        dtd = mlview_parsing_utils_load_a_dtd (sd, a_ctxt);
+
+        if (!dtd)
+                goto cleanup;
+
+        return dtd;
+
+ cleanup:
+        if (sd) {
+                mlview_ext_subs_def_destroy (sd);
+                sd = NULL;
+        }
+
+        if (dtd) {
+                xmlFreeDtd (dtd);
+                dtd = NULL;
+        }
+
+        return NULL;
+}
+
+/* About the use of mlview_parsing_utils_load_xml_file_with_dtd (x, NULL, y) in
+   functions which load schemas : I hope it works (I don't think there are DTD
+   specified in schemas :-) but user should not be prompted for a DTD when a 
+   schema is loaded, I think :-) */
+
+xmlRelaxNGPtr 
+mlview_parsing_utils_load_rng (const gchar *a_url,
+                               MlViewAppContext *a_ctxt)
+{
+        xmlDocPtr doc = NULL;
+        xmlRelaxNGPtr rng = NULL;
+        xmlRelaxNGParserCtxtPtr ctx = NULL;
+
+        g_return_val_if_fail (a_url, NULL);
+        g_return_val_if_fail (a_ctxt && 
+                              MLVIEW_IS_APP_CONTEXT (a_ctxt), 
+                              NULL);
+
+        doc = mlview_parsing_utils_load_xml_file_with_dtd ((gchar*)a_url, NULL, a_ctxt);
+
+        if (!doc)
+                goto cleanup;
+
+        ctx = xmlRelaxNGNewDocParserCtxt (doc);
+
+        if (!ctx)
+                goto cleanup;
+
+        rng = xmlRelaxNGParse (ctx);
+
+        if (!rng)
+                goto cleanup;
+
+        xmlFreeDoc (doc);
+        doc = NULL;
+
+        xmlRelaxNGFreeParserCtxt (ctx);
+        ctx = NULL;
+
+        return rng;
+
+ cleanup:        
+        if (doc) {
+                xmlFreeDoc (doc);
+                doc = NULL;
+        }
+
+        if (ctx) {
+                xmlRelaxNGFreeParserCtxt (ctx);
+                doc = NULL;
+        }
+
+        if (rng) {
+                xmlRelaxNGFree (rng);
+                rng = NULL;
+        }
+
+        return NULL;
+}
+
+xmlSchemaPtr
+mlview_parsing_utils_load_xsd (const gchar *a_url,
+                               MlViewAppContext *a_ctxt)
+{
+        xmlDocPtr doc = NULL;
+        xmlSchemaPtr xsd = NULL;
+        xmlSchemaParserCtxtPtr ctx = NULL;
+
+        g_return_val_if_fail (a_url, NULL);
+        g_return_val_if_fail (a_ctxt && 
+                              MLVIEW_IS_APP_CONTEXT (a_ctxt), 
+                              NULL);
+
+        doc = mlview_parsing_utils_load_xml_file_with_dtd ((gchar*)a_url, NULL, a_ctxt); 
+
+        if (!doc)
+                goto cleanup;
+
+        ctx = xmlSchemaNewDocParserCtxt (doc);
+
+        if (!ctx)
+                goto cleanup;
+
+        xsd = xmlSchemaParse (ctx);
+
+        if (!xsd)
+                goto cleanup;
+
+        xmlFreeDoc (doc);
+        doc = NULL;
+
+        xmlSchemaFreeParserCtxt (ctx);
+        ctx = NULL;
+
+        return xsd;
+
+ cleanup:        
+        if (doc) {
+                xmlFreeDoc (doc);
+                doc = NULL;
+        }
+
+        if (ctx) {
+                xmlSchemaFreeParserCtxt (ctx);
+                doc = NULL;
+        }
+
+        if (xsd) {
+                xmlSchemaFree (xsd);
+                xsd = NULL;
+        }
+
+        return NULL;
+}
+
+void
+mlview_parsing_utils_clean_dtd (xmlDtdPtr a_dtd)
+{
+        xmlNodePtr cur;
+
+        g_return_if_fail (a_dtd);
+
+        a_dtd->doc = NULL;
+        
+        cur = a_dtd->children;
+
+        while (cur) {
+                if (cur->doc) 
+                        cur->doc = NULL;
+
+                cur = cur->next;
+        }
+}
+
+enum MlViewStatus
+mlview_parsing_utils_serialize_node_to_buf (const xmlNode *a_node,
+                                            gchar **out_buf)
+{
+        enum MlViewStatus status = MLVIEW_OK ;
+        xmlBuffer *xml_buffer = NULL ;
+        xmlChar *content = NULL ;
+
+        g_return_val_if_fail (a_node, MLVIEW_BAD_PARAM_ERROR) ;
+
+        xml_buffer = xmlBufferCreate () ;
+        if (!xmlNodeDump (xml_buffer, a_node->doc, (xmlNode*)a_node, 0, 0)){
+                mlview_utils_trace_info ("No byte were writen during serialization") ;
+                status = MLVIEW_ERROR ;
+                goto cleanup ;
+        }
+        if (!xml_buffer->use) {
+                status = MLVIEW_ERROR ;
+                goto cleanup ;
+        }
+        content = g_strndup (xmlBufferContent (xml_buffer),
+                             xml_buffer->use) ;
+        if (!content) {
+                status = MLVIEW_OUT_OF_MEMORY_ERROR ;
+                goto cleanup ;
+        }
+        *out_buf = content ;
+        content = NULL ;
+        
+ cleanup:
+        if (xml_buffer) {
+                xmlBufferFree (xml_buffer) ;
+                xml_buffer = NULL ;
+        }
+        return status ;
+}
+
+enum MlViewStatus
+mlview_parsing_utils_do_comment_node (const xmlNode *a_node,
+                                      xmlNode **a_comment_node)
+{
+        enum MlViewStatus status = MLVIEW_OK ;
+        xmlNode *comment_node = NULL ;
+        gchar *buf = NULL ;
+        xmlChar *encoded_buf = NULL ;
+
+        g_return_val_if_fail (a_node 
+                              && a_node->doc 
+                              && a_comment_node,
+                              MLVIEW_BAD_PARAM_ERROR) ;
+
+        status = mlview_parsing_utils_serialize_node_to_buf (a_node, &buf) ;
+
+        if (status != MLVIEW_OK || !buf)
+                return status ;
+        encoded_buf = xmlEncodeEntitiesReentrant (a_node->doc,
+                                                  buf) ;
+        g_free (buf) ;
+        buf = NULL ;
+        comment_node = xmlNewDocComment (a_node->doc, encoded_buf) ;
+        if (!comment_node) {
+                status = MLVIEW_OUT_OF_MEMORY_ERROR ;
+                goto cleanup ;
+        }
+
+        *a_comment_node = comment_node ;
+        comment_node = NULL ;
+
+ cleanup:
+        if (buf) {
+                xmlMemFree (buf) ;
+                buf = NULL ;
+        }
+        if (comment_node) {
+                xmlFreeNode (comment_node) ;
+                comment_node = NULL ;
+        }
+        return status ;
+}
+
+enum MlViewStatus
+mlview_parsing_utils_parse_fragment (xmlDoc *a_doc,
+                                     const xmlChar *a_buf,
+                                     xmlNode **a_result_node)
+{
+        xmlDoc *doc = NULL ;
+        xmlNode *node = NULL ;
+        enum MlViewStatus status = MLVIEW_OK ;
+
+        doc = a_doc ;
+        if (!doc)
+                doc = xmlNewDoc ("1.0") ;
+        if (!doc) {
+                mlview_utils_trace_info ("Got a NULL document") ;
+                return MLVIEW_ERROR ;
+        }
+        if (xmlParseBalancedChunkMemory (doc, NULL, NULL, 0, 
+                                         a_buf, &node)) {
+                status = MLVIEW_ERROR ;
+                goto cleanup ;
+        }
+        
+        if (status == MLVIEW_OK) {
+                *a_result_node = node ;
+        }
+
+ cleanup:
+
+        if (doc && doc != a_doc) {
+                xmlFreeDoc (doc) ;
+                doc = NULL ;
+        }
+
+        return status ;
+}
