@@ -27,7 +27,7 @@
 #define __MLVIEW_TREE_VIEW_H__
 
 #include <libxml/tree.h>
-#include "mlview-tree-editor2.h"
+#include "mlview-tree-editor.h"
 #include "mlview-node-editor.h"
 #include "mlview-app-context.h"
 #include "mlview-view-adapter.h"
@@ -39,6 +39,7 @@
  */
 
 G_BEGIN_DECLS
+
 #define MLVIEW_TYPE_TREE_VIEW (mlview_tree_view_get_type ())
 #define MLVIEW_TREE_VIEW(widget) (G_TYPE_CHECK_INSTANCE_CAST ((widget), MLVIEW_TYPE_TREE_VIEW, MlViewTreeView))
 #define MLVIEW_TREE_VIEW_CLASS(widget) (G_TYPE_CHECK_CLASS_CAST ((widget), MLVIEW_TYPE_TREE_VIEW, MlViewTreeViewClass))
@@ -62,8 +63,7 @@ struct _MlViewTreeViewClass {
         MlViewViewAdapterClass parent_class;
 
         /*signal default handlers */
-        void (*document_changed) (MlViewTreeView *
-                                  tree_view, gpointer a_data);
+        void (*document_changed) (MlViewTreeView *tree_view, gpointer a_data);
 };
 
 GType mlview_tree_view_get_type (void);
@@ -75,7 +75,10 @@ GtkWidget *mlview_tree_view_new (MlViewXMLDocument *a_doc,
 void mlview_tree_view_set_app_context (MlViewTreeView * a_tree_view,
                                        MlViewAppContext * a_app_ctxt);
 
-MlViewTreeEditor2 * mlview_tree_view_get_tree_editor (MlViewTreeView * a_tree_view);
+enum MlViewStatus mlview_tree_view_get_app_context (MlViewTreeView *a_this,
+                                                    MlViewAppContext **a_app_context_ptr) ;
+
+MlViewTreeEditor * mlview_tree_view_get_current_tree_editor (MlViewTreeView * a_tree_view);
 
 MlViewNodeEditor * mlview_tree_view_get_node_editor (MlViewTreeView * a_tree_view);
 
@@ -87,16 +90,38 @@ void mlview_tree_view_construct (MlViewTreeView * a_tree_view,
                                  gchar * a_name,
                                  MlViewAppContext *a_app_ctxt) ;
 
+enum MlViewStatus mlview_tree_view_create_internal_subset_node_interactive (MlViewTreeView *a_this) ;
+
+void mlview_tree_view_add_child_element_node (MlViewTreeView *a_this,
+                                              const gchar *a_element_name) ;
+
+void mlview_tree_view_add_child_text_node (MlViewTreeView *a_this,
+                                           const gchar *a_text) ;
+
 void mlview_tree_view_add_child_node_interactive (MlViewTreeView * a_this);
 
 void mlview_tree_view_insert_sibling_node_interactive (MlViewTreeView * a_this);
 
 void mlview_tree_view_insert_prev_sibling_node_interactive (MlViewTreeView * a_this);
 
+void mlview_tree_view_insert_prev_sibling_text_node (MlViewTreeView *a_this,
+                                                     const gchar *a_text) ;
+
+void mlview_tree_view_insert_next_sibling_text_node (MlViewTreeView *a_this,
+                                                     const gchar *a_text) ;
+
+void mlview_tree_view_insert_prev_sibling_element_node (MlViewTreeView *a_this,
+                                                        const gchar *a_element_name) ;
+
+void mlview_tree_view_insert_next_sibling_element_node (MlViewTreeView *a_this,
+                                                        const gchar *a_element_name) ;
+
 void mlview_tree_view_insert_next_sibling_node_interactive (MlViewTreeView * a_this);
 
 void mlview_tree_view_cut_node (MlViewTreeView * a_this);
 
+void mlview_tree_view_comment_current_node (MlViewTreeView *a_this) ;
+
 void mlview_tree_view_copy_node (MlViewTreeView * a_this);
 
 void mlview_tree_view_paste_node_as_child (MlViewTreeView * a_this);
@@ -115,16 +140,33 @@ void mlview_tree_view_set_main_paned_proportions (MlViewTreeView * a_this,
 
 void mlview_tree_view_set_all_paned_proportions (MlViewTreeView * a_this,
                                                  const guint a_main_paned_percentage,
-                                                 const guint
-                                                 a_node_editor_paned_percentage);
+                                                 const guint a_node_editor_paned_percentage);
 
 void mlview_tree_view_set_xml_document_path (MlViewTreeView * a_view,
                                              gchar * a_file_path);
 
 void mlview_tree_view_expand_tree_to_depth_interactive (MlViewTreeView * a_view);
 
+enum MlViewStatus mlview_tree_view_get_contextual_menu (MlViewTreeView *a_this, 
+                                                        GtkWidget **a_menu_ptr) ;
+
+enum MlViewStatus mlview_tree_view_get_edit_menu_for_application (MlViewTreeView *a_this,
+                                                                  GtkWidget **a_menu_ptr) ;
+
 void mlview_tree_view_update_contextual_menu (MlViewTreeView * a_view,
                                               GtkMenu ** a_menu_ptr);
 
+enum MlViewStatus mlview_tree_view_focus_on_node_editor (MlViewTreeView *a_view) ;
+
+enum MlViewStatus mlview_tree_view_focus_on_tree_editor (MlViewTreeView *a_view) ;
+
+void mlview_tree_view_select_parent_node (MlViewTreeView *a_this) ;
+
+void mlview_tree_view_select_prev_sibling_node (MlViewTreeView *a_this) ;
+
+void mlview_tree_view_select_next_sibling_node (MlViewTreeView *a_this) ;
+
+GtkUIManager * mlview_tree_view_get_ui_manager (MlViewTreeView *a_this) ;
+
 G_END_DECLS
 #endif
