@@ -50,15 +50,11 @@ struct _MlViewViewAdapterPrivate {
 
 /*signals*/
 enum {
-        NAME_CHANGED,
         /*must be the last one! */
         NUMBER_OF_SIGNALS
 };
 
 static GtkVBoxClass *parent_class = NULL;
-static gint
-        gv_mlview_view_adapter_signals[NUMBER_OF_SIGNALS] =
-        { 0 };
 
 
 static void mlview_view_adapter_class_init (MlViewViewAdapterClass * a_klass);
@@ -66,7 +62,7 @@ static void mlview_view_adapter_class_init (MlViewViewAdapterClass * a_klass);
 static void mlview_view_adapter_init (MlViewViewAdapter * a_mlview_xml_doc);
 
 static enum MLVIEW_VIEW_ADAPTER_STATUS mlview_view_adapter_connect_to_document (MlViewViewAdapter * a_tree_view,
-                                                                                          MlViewXMLDocument * a_xml_doc);
+                                                                                MlViewXMLDocument * a_xml_doc);
 
 static enum MLVIEW_VIEW_ADAPTER_STATUS mlview_view_adapter_disconnect_from_document (MlViewViewAdapter * a_this,
                                                                                                MlViewXMLDocument * a_xml_doc);
@@ -133,19 +129,6 @@ mlview_view_adapter_class_init (MlViewViewAdapterClass *a_klass)
 
         gobject_class->dispose = mlview_view_adapter_dispose ;
         gobject_class->finalize = mlview_view_adapter_finalize ;
-
-        /*now, create the signals */
-        gv_mlview_view_adapter_signals[NAME_CHANGED] =
-                g_signal_new ("name-changed",
-                              G_TYPE_FROM_CLASS (gobject_class),
-                              GTK_RUN_FIRST,
-                              GTK_SIGNAL_OFFSET
-                              (MlViewViewAdapterClass,
-                               name_changed), NULL, NULL,
-                              gtk_marshal_NONE__NONE,
-                              GTK_TYPE_NONE, 0, NULL);
-        a_klass->name_changed = NULL;
-        
 }
 
 
@@ -240,14 +223,9 @@ mlview_view_adapter_set_name (MlViewIView *a_this,
                 g_free (PRIVATE (adapter)->view_name);
                 PRIVATE (adapter)->view_name = NULL;
         }
-
         if (a_name)
                 PRIVATE (adapter)->view_name =
                         g_strdup (a_name);
-
-        gtk_signal_emit (GTK_OBJECT (adapter),
-                         gv_mlview_view_adapter_signals
-                         [NAME_CHANGED]);
         return MLVIEW_OK ;
 }
 
@@ -434,7 +412,7 @@ mlview_view_adapter_init (MlViewViewAdapter *a_this)
  *@param a_xml_doc the document which signals are to be connected to.
  *@return the status.
  */
-static enum MlViewStatus 
+static  enum MLVIEW_VIEW_ADAPTER_STATUS
 mlview_view_adapter_connect_to_document (MlViewViewAdapter * a_this,
                                          MlViewXMLDocument * a_xml_doc) 
 {
@@ -615,6 +593,9 @@ mlview_view_adapter_finalize (GObject *a_view)
         }
 }
 
+/*
+   NOT USED ATM
+ *
 static enum MlViewStatus
 mlview_view_adapter_set_name_interactive2 (MlViewIView *a_this)
 {
@@ -693,7 +674,7 @@ mlview_view_adapter_set_name_interactive2 (MlViewIView *a_this)
         }
         return status ;
 }
-
+*/
 /*********************************************
  *Public methods of this class.
  *********************************************/
@@ -818,7 +799,7 @@ mlview_view_adapter_get_type (void)
                 type_id = g_type_register_static (GTK_TYPE_VBOX,
                                                   "MlViewViewAdapter",
                                                   &type_info, 0);
-                g_type_add_interface_static (type_id, 
+                g_type_add_interface_static (type_id,
                                              MLVIEW_TYPE_IVIEW,
                                              &iview_info) ;
         }
